/* ==============================================================================

System Name:    Signal Generator Module demo 

File Name:      PROFILE.C

Description:    Primary System file for demonstrating the Signal Generator module

Originator:     Digital control systems Group - Texas Instruments

Target dependency:  C28x

Description:
============

   The "C" frame work, for demonstrating the signal generator module is given below
                 ___________           _____________
                |           |         |             |
                |           | X1=OUT  |   EVMDAC    |
                | PROFILE   |-------->|   DLOG_4CH  |
                |           |         |   PWMDAC    |
                |___________|         |_____________|

*/


#include <sgen.h>
#include <dlog4ch.h>

/* Create an instance of DATALOG Module             */
DLOG_4CH dlog=DLOG_4CH_DEFAULTS;      
    
/* Create an instance of Signal generator module    */
PROFILE sgen=PROFILE_DEFAULTS;
         
/* Temp variable to store the signal genr output    */   
int x1;
     
void main()
{    

/* DATALOG module initialisation */     
       dlog.iptr1=&x1;
       dlog.trig_value=-30000;
       dlog.prescalar=1;
       dlog.size=0x400;     
       dlog.init(&dlog);       

/* Signal Generator module initialisation   */ 
      sgen.mode=1;              /* Set Mode bit for Continuous signal Generation  */            
      sgen.prescalar=1;         
      sgen.freq=5369;           /* freq = (Required Freq/Max Freq)*2^15           */
                                /*      = (50/305.17)*2^15 = 5369                 */    
      sgen.step_max=4000;       /* Max Freq= (step_max * sampling freq)/(4*65536) */
                                /* Max Freq = (4000*20k)/(4*65536) = 305.17       */
      sgen.gain=0x7fff;         /* ~1 in Q15 format                               */
      sgen.offset=0;
      sgen.t_rmpup=51;          /* 20% of T, 0.2 in Q8  */
      sgen.t_max=77;            /* 30% of T, 0.3 in Q8  */
      sgen.t_rmpdn=102;         /* 40% of T, 0.4 in Q8  */
      sgen.t_min=25;            /* 10% of T, 0.1 in Q8 */
      sgen.init(&sgen);
                

/*---------------------------------------------------------------------------
    Nothing running in the background at present           
----------------------------------------------------------------------------*/

        while(1) 
        {
            sgen.calc(&sgen);
            x1=sgen.out;
            dlog.update(&dlog);
        }       

} /* End: main() */

             




