/* ==============================================================================

System Name:    Signal Generator Module demo 

File Name:      SGENT1.C

Description:    Primary System file for demonstrating the Signal Generator module

Originator:     Digital control systems Group - Texas Instruments

Target dependency:  C28x

Description:
============

   The "C" frame work, for demonstrating the signal generator module is given below
                 ___________           _____________
                |           |         |             |
                |           | X1=OUT  |   EVMDAC    |
                |  SGENT_1  |-------->|   DLOG_4CH  |
                |           |         |   PWMDAC    |
                |___________|         |_____________|

*/

#include <sgen.h>
#include <dlog4ch.h>

/* Create an instance of DATALOG Module             */
DLOG_4CH dlog=DLOG_4CH_DEFAULTS;      
    
/* Create an instance of Signal generator module    */
SGENT_1 sgen = SGENT_1_DEFAULTS;
         
/* Temp variable to store the signal genr output    */   
int x1;
     
void main()
{    

/* DATALOG module initialisation                    */        
       dlog.iptr1=&x1;
       dlog.trig_value=0x800;
       dlog.size=0x400;         /* Can log 1024 Samples                      */
       dlog.init(&dlog);

/* Signal Generator module initialisation           */ 
       sgen.offset=0;
       sgen.gain=0x7fff;        /* gain=1 in Q15                              */
       sgen.freq=5369;          /* freq = (Required Freq/Max Freq)*2^15       */
                                /*      = (50/305.17)*2^15 = 5369             */         
       sgen.step_max=1000;      /* Max Freq= (step_max * sampling freq)/65536 */
                                /* Max Freq = (1000*20k)/65536 = 305.17       */
              

/*---------------------------------------------------------------------------
    Nothing running in the background at present           
----------------------------------------------------------------------------*/

        while(1) 
        {
            sgen.calc(&sgen);
            x1=sgen.out;
            dlog.update(&dlog);
        }       

} /* End: main() */

             




