/* ==============================================================================

System Name:    Signal Generator Module demo 

File Name:      SGENT3D.C

Description:    Primary System file for demonstrating the Signal Generator module

Originator:     Digital control systems Group - Texas Instruments

Target dependency:  C28x

Description:
============

   The "C" frame work, for demonstrating the signal generator module is given below
                             ___________________                     __________________
                            |                   |     x11=OUT11     |                  |
                            |                   |------o----------->|                  |
                            |                   |     x12=OUT12     |                  |
        phase   o---------->|                   |------o----------->|                  |
                            |                   |     x13=OUT13     |   PWMDAC         |
        gain    o---------->|                   |------o----------->|                  |
                            |                   |                   |   EVMDAC         |
        offset  o---------->|     SGENT_3D      |     x21=OUT21     |                  |
                            |                   |----o------------->|   DLOG_4CH       |
        freq    o---------->|                   |     x22=OUT22     |                  |
                            |                   |----o------->      |                  |
                            |                   |     x23=OUT23     |                  |
                            |                   |----o------->      |                  |
                            |                   |                   |                  |
                            |___________________|                   |__________________|

*/

#include <sgen.h>
#include <dlog4ch.h>

/* Create an instance of DATALOG Module             */
DLOG_4CH dlog=DLOG_4CH_DEFAULTS;      
    
/* Create an instance of Signal generator module    */
SGENT_3D sgen = SGENT_3D_DEFAULTS;
         
/* Temp variable to store the signal genr output */   
int x11,x12,x13,x21,x22,x23 ;
     
void main()
{    

/* DATALOG module initialisation    */        
       dlog.iptr1=&x11;  
       dlog.iptr2=&x12; 
       dlog.iptr3=&x13; 
       dlog.iptr4=&x21; 
       dlog.trig_value=0x800;
       dlog.size=0x400;         /* Can log 1024 Samples                     */
       dlog.init(&dlog);

/* Signal Generator module initialisation   */ 
       sgen.offset=0;
       sgen.gain=0x7fff;        /* gain=1 in Q15                              */
       sgen.freq=5369;          /* freq = (Required Freq/Max Freq)*2^15       */
                                /*      = (50/305.17)*2^15 = 5369             */         
       sgen.step_max=1000;      /* Max Freq= (step_max * sampling freq)/65536 */
                                /* Max Freq = (1000*20k)/65536 = 305.17       */
       sgen.phase=0x4000;       /* Phase= (required Phase)/180 in Q15 format  */    
                                /*      =  (+90/180) in Q15 = 4000h           */
        
    
              

/*---------------------------------------------------------------------------
    Nothing running in the background at present           
----------------------------------------------------------------------------*/

        while(1) 
        {
            sgen.calc(&sgen);
            x11=sgen.out11; 
            x12=sgen.out12;
            x13=sgen.out13;
            x21=sgen.out21; 
            x22=sgen.out22;
            x23=sgen.out23;
            dlog.update(&dlog);
        }       

} /* End: main() */

             




