/* ==============================================================================

System Name:    Signal Generator Module demo 

File Name:      SGENTI3.C

Description:    Primary System file for demonstrating the Signal Generator module

Originator:     Digital control systems Group - Texas Instruments

Target dependency:  x2407/x241/2/3

Description:
============

   The "C" frame work, for demonstrating the signal generator module is given below
                             ___________________                   ______________
                            |                   |     x1=out1     |              |
                            |                   |------o--------->|  EVMDAC      |
         gain   o---------->|                   |     x2=out2     |  PWMDAC      |
         offset o---------->|     SGENTI_3      |------o--------->|              |
         freq   o---------->|                   |     x3=out3     |  DLOG_4CH    |
                            |                   |------o--------->|              | 
                            |___________________|                 |______________|

*/


#include <sgen.h>
#include <dlog4ch.h>

/* Create an instance of DATALOG Module             */
DLOG_4CH dlog=DLOG_4CH_DEFAULTS;      
    
/* Create an instance of Signal generator module    */
SGENTI_3 sgen = SGENTI_3_DEFAULTS;
         
/* Temp variable to store the signal genr output    */   
int x1,x2,x3;
     
void main()
{    

/* DATALOG module initialisation    */        
       dlog.iptr1=&x1;  
       dlog.iptr2=&x2;
       dlog.iptr3=&x3;
       dlog.trig_value=0x800;
       dlog.size=0x400;         /* Can log 1024 Samples                     */
       dlog.init(&dlog);

/* Signal Generator module initialisation   */ 
       sgen.offset=0;
       sgen.gain=0x7fff;        /* gain=1 in Q15                              */
       sgen.freq=5369;          /* freq = (Required Freq/Max Freq)*2^15       */
                                /*      = (50/305.17)*2^15 = 5369             */         
       sgen.step_max=1000;      /* Max Freq= (step_max * sampling freq)/65536 */
                                /* Max Freq = (1000*20k)/65536 = 305.17       */
       
              

/*---------------------------------------------------------------------------
    Nothing running in the background at present           
----------------------------------------------------------------------------*/

        while(1) 
        {
            sgen.calc(&sgen);
            x1=sgen.out1; 
            x2=sgen.out2;
            x3=sgen.out3;
            dlog.update(&dlog);
        }       

} /* End: main() */

             




