/* ==============================================================================

System Name:    Signal Generator Module demo 

File Name:      TZDLGEN.C

Description:    Primary System file for demonstrating the Signal Generator module

Originator:     Digital control systems Group - Texas Instruments

Target dependency:  C28x

Description:
============

   The "C" frame work, for demonstrating the signal generator module is given below
                 ___________           _____________
                |           |         |             |
                |           | X1=OUT  |   EVMDAC    |
                | TZDLGEN   |-------->|   DLOG_4CH  |
                |           |         |   PWMDAC    |
                |___________|         |_____________|

*/


#include <sgen.h>
#include <dlog4ch.h>

/* Create an instance of DATALOG Module             */
DLOG_4CH dlog=DLOG_4CH_DEFAULTS;      
    
/* Create an instance of Signal generator module    */
TZDLGEN tgen = TZDLGEN_DEFAULTS;
         
/* Temp variable to store the signal genr output    */   
int x1;
     
void main()
{    

/* DATALOG module initialisation */     
       dlog.iptr1=&x1;
       dlog.trig_value=0x800;
       dlog.prescalar=1;
       dlog.size=0x400;     
       dlog.init(&dlog);       

/* Signal Generator module initialisation   */ 
      tgen.prescalar=1;         
      tgen.freq=5369;           /* freq = (Required Freq/Max Freq)*2^15           */
                                /*      = (50/305.17)*2^15 = 5369                 */    
      tgen.step_max=4000;       /* Max Freq= (step_max*sampling freq)/(4*65536*prescalar) */
                                /* Max Freq = (4000*20k)/(4*65536*1) = 305.17     */
      tgen.gain=0x7fff;         /* ~1 in Q15 format                               */
      tgen.offset=0;
      tgen.init(&tgen);
  
                

/*---------------------------------------------------------------------------
    Nothing running in the background at present           
----------------------------------------------------------------------------*/

        while(1) 
        {
            tgen.calc(&tgen);
            x1=tgen.out;
            dlog.update(&dlog);
        }       

} /* End: main() */

             




